package it.finanze.dogane.accise.aautotrasportatori.model;

import it.finanze.dogane.accise.aautotrasportatori.config.Contatori;
import it.finanze.dogane.accise.aautotrasportatori.ui.AutotrasportatoriFrame;
import it.finanze.dogane.accise.model.AbstractRecord;

import java.io.IOException;
import java.math.BigInteger;

public abstract class AutotrasportatoriRecord extends AbstractRecord {
	
	private static final String valoreFisso = "CAR100";
	private BigInteger annoDic = BigInteger.ZERO;
	private String partitaIva = "";
	private BigInteger progPartitaIva = BigInteger.valueOf(0);
	private BigInteger tipoRecord = BigInteger.valueOf(0);
	
	private static final int dimValoreFisso = 6;
	private static final int dimAnnoDic =4;
	private static final int dimPartitaIvaItaliana =12;
	private static final int dimProgPartitaIva =2;
	private static final int dimTipoRecord =2;

	public void impostaCampiComuni(AbstractRecord in) {
		AutotrasportatoriRecord erin = (AutotrasportatoriRecord)in;
		partitaIva = erin.getPartitaIva();
		progPartitaIva = erin.getProgPartitaIva();
		annoDic = erin.getAnnoDic();
	}



	public void scriviOggetto(java.io.OutputStream out) throws IOException {
		
		super.scriviOggetto(out);
		
		out.write(normalizzaCampo(valoreFisso, dimValoreFisso));
		out.write(normalizzaCampo(annoDic, dimAnnoDic));
		out.write(normalizzaCampo(partitaIva, dimPartitaIvaItaliana));
		out.write(normalizzaCampo(progPartitaIva, dimProgPartitaIva));
		out.write(normalizzaCampo(tipoRecord, dimTipoRecord));
		
	}
	
	public void leggiOggetto(java.io.InputStream in) throws IOException {

		super.leggiOggetto(in);
		//Contatori.ANNO_IN_LETTURA=2007;
		readFieldString(in, dimValoreFisso);
		annoDic = readFieldBigInteger(in, dimAnnoDic);
		try{
			/*int anno=Integer.parseInt(annoDic.toString());
			if(anno<2006) throw new Exception("");
			if(anno==2006) Contatori.ANNO_IN_LETTURA=2006;*/
			
			if(annoDic.intValue() != new Integer(AutotrasportatoriFrame.getAnnoRiferimento()).intValue())
				throw new Exception("");
				
		}catch (NumberFormatException nfEx){
			throw new IOException("Impossibile Aprire il File: File Danneggiato. (Err.Code: NFE)");
		}
		 catch(Exception e){
			throw new IOException("Impossibile aprire la dichiarazione di anni precedenti al "+AutotrasportatoriFrame.getAnnoRiferimento());
		}
		partitaIva = readFieldString(in, dimPartitaIvaItaliana);
		progPartitaIva = readFieldBigInteger(in, dimProgPartitaIva);
		tipoRecord = readFieldBigInteger(in, dimTipoRecord);	
			
	}

	/**
	 * @return
	 */
	public static String getValoreFisso() {
		return valoreFisso;
	}


	/**
	 * @return
	 */
	
	/**
	 * @return
	 */
	public static int getDimAnnoDic() {
		return dimAnnoDic;
	}

	/**
	 * @return
	 */
	public static int getDimPartitaIvaItaliana() {
		return dimPartitaIvaItaliana;
	}

	/**
	 * @return
	 */
	public static int getDimProgPartitaIva() {
		return dimProgPartitaIva;
	}

	/**
	 * @return
	 */
	public static int getDimTipoRecord() {
		return dimTipoRecord;
	}

	/**
	 * @return
	 */
	public static int getDimValoreFisso() {
		return dimValoreFisso;
	}


	/**
	 * @return
	 */
	public BigInteger getProgPartitaIva() {
		return progPartitaIva;
	}

	/**
	 * @return
	 */
	public BigInteger getTipoRecord() {
		return tipoRecord;
	}

	/**
	 * @param integer
	 */
	public void setProgPartitaIva(BigInteger integer) {
		progPartitaIva = integer;
	}

	/**
	 * @param integer
	 */
	public void setTipoRecord(BigInteger integer) {
		tipoRecord = integer;
	}

	/**
	 * @return
	 */
	public BigInteger getAnnoDic() {
		return annoDic;
	}

	/**
	 * @param integer
	 */
	public void setAnnoDic(BigInteger integer) {
		annoDic = integer;
	}

	/**
	 * @return
	 */
	public String getPartitaIva() {
		return partitaIva;
	}

	/**
	 * @param string
	 */
	public void setPartitaIva(String string) {
		partitaIva = string;
	}
	
	//implementato nelle sottoclassi
	public Object[] getQuadroToArray(){return null;};		

}
