package it.finanze.dogane.accise.aautotrasportatori.model;

import it.finanze.dogane.accise.aautotrasportatori.ui.AutotrasportatoriFrame;
import it.finanze.dogane.accise.aautotrasportatori.ui.quadri.Quadro;
import it.finanze.dogane.accise.aautotrasportatori.util.QuadroAutotrasportatoriUtil;
import it.finanze.dogane.accise.model.DicInterface;
import it.finanze.dogane.accise.model.Prospect;
import it.finanze.dogane.accise.util.ComparatorListModel;
import it.finanze.dogane.accise.util.QuadroUtil;
import it.finanze.dogane.accise.util.ResourceAccess;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

public class ProspettoConsumo implements Serializable, Prospect {
	
	private ResourceAccess resourceAccess = new ResourceAccess(this);
	private Frontespizio frontespizio =  new Frontespizio();
	private List listaRecord = new LinkedList();
	private List listaDatiContabili = new ArrayList();
	private int[] listaAliquote = new int[AutotrasportatoriFrame.getNumeroAliquote()];
	
	public void scriviOggetto(java.io.OutputStream out) throws IOException {
		
		updateRecordNumbers();
		frontespizio.scriviOggetto(out);
		
		//solo i dati contabili valorizzati
		int riga = 0;
		for (int i = 0; i < listaAliquote.length; i++) {
			
			riga++;						
		  	BigInteger litri = frontespizio.getArrayLitriConsumati()[i];
		  	if(litri.toString().equalsIgnoreCase("") || litri.toString().equalsIgnoreCase("0") )
		  		continue;		  	
				
			BigDecimal importi = frontespizio.getArrayImportoACredito()[i]; 
		    DatiContabiliRecord record = new DatiContabiliRecord(litri,importi,new BigInteger(""+riga));
			record.setTipoRecord(new BigInteger(""+RecordFactory.DATI_CONTABILI_TYPE));
			record.impostaCampiComuni(frontespizio);
			record.scriviOggetto(out);
		}
		
		Collections.sort(listaRecord, new ComparatorListModel());
		Iterator it = listaRecord.iterator();
		while(it.hasNext()) {
			AutotrasportatoriRecord record = (AutotrasportatoriRecord)it.next();
			record.impostaCampiComuni(frontespizio);
			record.scriviOggetto(out);
		}
	}

	public void leggiOggetto(java.io.InputStream in) throws IOException {

		frontespizio = new Frontespizio();
		frontespizio.initArrayLitriImporti(AutotrasportatoriFrame.getNumeroAliquote());
		listaRecord.clear();	

		frontespizio.leggiOggetto(in);				
		
		BufferedReader reader = new BufferedReader(new InputStreamReader(in));
		AutotrasportatoriRecord record = RecordFactory.readNextRecord(reader);
		
		while(record != null) {
			
			if( !(record instanceof AutotrasportatoriInvalidRecord) )			
				if(  !(record instanceof AutotrasportatoriQuadroA) ||
					((record instanceof AutotrasportatoriQuadroA)
				    && (record!=null) 
					/*&& !((AutotrasportatoriQuadroA)record).getTipoQuadroA().equals("2")*/)
				)
				
				
				if(record instanceof DatiContabiliRecord)
					listaDatiContabili.add(record);
				else
					listaRecord.add(record);
				
			record = RecordFactory.readNextRecord(reader);
			
		}
		
		reader.close();
	}

	protected void updateRecordNumbers() {
		
		frontespizio.setTotaleFileRecord(BigInteger.valueOf(listaRecord.size() + 1));
		frontespizio.setTotaleRecordQuadroA(QuadroUtil.countRecord(listaRecord, BigInteger.valueOf(10)));
		frontespizio.setTotaleRecordQuadroC(QuadroUtil.countRecord(listaRecord, BigInteger.valueOf(30)));
		frontespizio.setFlagQuadroB(QuadroUtil.countRecord(listaRecord, BigInteger.valueOf(20)));
	}


	public Object clone() throws CloneNotSupportedException {
		ProspettoConsumo clonato = new ProspettoConsumo();
	
		clonato.setResourceAccess(this.resourceAccess);
		clonato.setFrontespizio(this.frontespizio);
		clonato.setListaRecord(this.listaRecord);
	
		return clonato;
	}
	
	public Frontespizio getFrontespizio() {
		return frontespizio;
	}

	/**
	 * @param frontespizio
	 */
	public void setFrontespizio(Frontespizio frontespizio) {
		this.frontespizio = frontespizio;
	}


	public List getListaQuadro(int[] tipologieQuadro) {
		return QuadroUtil.getListaQuadro(listaRecord, tipologieQuadro);
	}
	
	// aggiunto reply
	// se tipoQuadroA  null agisce come getListaQuadro(int[] tipologieQuadro)
	public List getListaQuadro(int[] tipologieQuadro, String tipoQuadroA) {
		
		if( tipoQuadroA == null)		
			return QuadroUtil.getListaQuadro(listaRecord, tipologieQuadro);
		else
			return getListaQuadroA(tipoQuadroA);
			
	}
	
	// aggiunto reply
	// prende tutti i record del quadro a del tipo specificato
	public List getListaQuadroA( String tipoQuadroA ){
		
		List listaQuadriA= QuadroUtil.getListaQuadro(listaRecord, Quadro.QUADRO_A );// ha lo stesso valore di A2
				
				
				
		List listaRecordDaRimuovere = new LinkedList();

		for( int i=0; i < listaQuadriA.size() ; i++   ){
			
			AutotrasportatoriQuadroA quadroArecord = (AutotrasportatoriQuadroA)listaQuadriA.get(i);
			if( !quadroArecord.getTipoQuadroA().equals(tipoQuadroA) )
				listaRecordDaRimuovere.add( quadroArecord );
		}
		
		listaQuadriA.removeAll(listaRecordDaRimuovere);
		
		return listaQuadriA;
	}

	public void setListaQuadro(List listaQuadro, int[] tipologieQuadro) {
		QuadroUtil.removeListaQuadro(listaRecord, tipologieQuadro);
		QuadroUtil.addListaQuadro(listaRecord, listaQuadro);
	}
	
	// aggiunto reply
	public void setListaQuadroA(List listaQuadro, String tipologiaQuadroA ) {
		
		QuadroAutotrasportatoriUtil.removeListaQuadroA( listaRecord, tipologiaQuadroA );
		QuadroUtil.addListaQuadro( listaRecord, listaQuadro );
		
	}

	/**
	 * @return
	 */
	public ResourceAccess getResourceAccess() {
		return resourceAccess;
	}

	/**
	 * @param access
	 */
	public void setResourceAccess(ResourceAccess access) {
		resourceAccess = access;
	}

	public Object scriviOggetto(OutputStream out, int size) throws IOException {
		throw new UnsupportedOperationException();
	}

	public void setListaRecord(List listaRec){
		this.listaRecord=listaRec;
	}
	public List getListaRecord(){
		return this.listaRecord;
	}

	public Object scriviOggettoInterchange() throws IOException {
		throw new UnsupportedOperationException();
	}

	/**
	 * @return
	 */
	public List getListaDatiContabili() {
		return listaDatiContabili;
	}

	/**
	 * @param list
	 */
	public void setListaDatiContabili(List list) {
		listaDatiContabili = list;
	}

}
