package it.finanze.dogane.accise.combo; 

import it.finanze.dogane.accise.util.StringUtil;

import java.io.Serializable;

/**
 * Questa classe ha due attributi (codice e descrizione)
 * che sono comuni a molti value objects. Tutti gli oggetti
 * che hanno questi due attributi dovrebbero estendere da questo oggetto.
 * In questo modo si possono utilizzare anche i metodi per combinare i
 * due attributi in una sola stringa e viceversa.
 */
public abstract class AbstractTipo implements Serializable{

	
	private String _codice = null;
	private String _descrizione = null;

	public abstract String getSeparator();

	/**
	 * @return
	 */
	public String getCodice() {
		return _codice;
	}

	/**
	 * @return
	 */
	public String getDescrizione() {
		return _descrizione;
	}

	/**
	 * @param string
	 */
	public void setCodice(String string) {
		_codice = string;
	}

	/**
	 * @param string
	 */
	public void setDescrizione(String string) {
		_descrizione = string;
	}

	/**
	 * ritorna una stringa che  la concatenazione di codice e descrizione separati
	 * dal SEPARATOR definito. 
	 * @return
	 */
	public String getDescrizioneEstesa() {
		return StringUtil.denullify(_codice) + getSeparator() + StringUtil.denullify(_descrizione);
	}

	/**
	 * @param string
	 */
	public void setDescrizioneEstesa(String string) {
		if(string != null) {
			int indexOf = string.indexOf(getSeparator());
			if(indexOf == -1) {
				_codice = string;
				_descrizione = null;
			} else {
				_codice = string.substring(0, indexOf);
				_descrizione = string.substring(indexOf + getSeparator().length());
			}
		} else {
			_codice = null;
			_descrizione = null;
		}
	}

	public String toString() {
		return getDescrizioneEstesa();
	}


	public boolean equals(Object obj) {
		boolean retvalue = false;
		if(obj != null) {
			if(this.getClass().equals(obj.getClass())) {
				AbstractTipo other = (AbstractTipo)obj;
				if(getCodice() == null) {
					if(other.getCodice() == null) {
						retvalue = true;
					} else {
						retvalue = false;
					}
				} else {
					retvalue = getCodice().equals(other.getCodice());
				}
			} else {
				retvalue = false;
			}
		}

		return retvalue;
	}
	
	


	public int hashCode() {
		int retvalue = 0;
		if(getCodice() != null) {
			retvalue = getCodice().hashCode();
		}
		return retvalue;
	}

}
