package it.finanze.dogane.accise.model;

import it.finanze.dogane.accise.util.StringUtil;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.Serializable;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.StringTokenizer;

public abstract class AbstractRecord implements Serializable, DicInterface {

	public static final byte[] FINERECORD ={0x0D, 0x0A};
	
	public static final String PLUS = "+";
	public static final String MINUS = "-";

	public abstract BigInteger getTipoRecord();

	public void scriviOggetto(java.io.OutputStream out) throws IOException {
	}

	public void leggiOggetto(java.io.InputStream in) throws IOException {
	}

	public abstract void impostaCampiComuni(AbstractRecord in);


	protected String readFieldString(java.io.InputStream in, int dimensione) throws IOException {
		byte[] field = new byte[dimensione];
		int bytesRead = in.read(field);
		
		String retvalue = new String(field).trim();
		return retvalue;
	}

	/**
	 * @deprecated usate al posto {@link #readFieldBigInteger()}
	 * @param in
	 * @param dimensione
	 * @return
	 * @throws IOException
	 */
	protected int readFieldInt(java.io.InputStream in, int dimensione) throws IOException {
		return Integer.parseInt(readFieldString(in, dimensione));
	}

	protected BigInteger readFieldBigInteger(java.io.InputStream in, int dimensione) throws IOException {
		String value = readFieldString(in, dimensione).toString().trim();
		if(StringUtil.nullify(value)==null){
			return  BigInteger.valueOf(0);
		}else{
			return new BigInteger(value);
		}
	}
	
	/**
	 * @deprecated usate al posto {@link #readFieldBigDecimal()}
	 * @param in
	 * @param dimensioneInt
	 * @param dimensioneDec
	 * @return
	 * @throws IOException
	 */
	protected double readFieldDouble(java.io.InputStream in, int dimensioneInt,int dimensioneDec) throws IOException {
		String parteIntera = readFieldString(in, dimensioneInt);
		String parteDecimale = readFieldString(in, dimensioneDec);
		
		return Double.parseDouble(parteIntera + "." + parteDecimale);
	}

	protected BigDecimal readFieldBigDecimal(java.io.InputStream in, int dimensioneInt,int dimensioneDec) throws IOException {
		String parteIntera = readFieldString(in, dimensioneInt);
		String parteDecimale = readFieldString(in, dimensioneDec);
		
		boolean existsZeroToBeRemoved = true;
		while(existsZeroToBeRemoved) {
			int lastIndexOf = parteDecimale.lastIndexOf("0");
			int length = parteDecimale.length();
			if(lastIndexOf != -1 && lastIndexOf == length - 1 && length > 1) {
				parteDecimale = parteDecimale.substring(0, parteDecimale.length()-1 );				 
			} else {
				existsZeroToBeRemoved = false;
			}
		}
		
		//String value = parteIntera + "." + parteDecimale;
		if(parteIntera.equals("")){
			return BigDecimal.valueOf(0,0);
		}else{
			BigDecimal parteInteraBigDec = new BigDecimal(parteIntera);
			BigDecimal parteDecimaleBigDec = new BigDecimal("0." + parteDecimale);

			return parteInteraBigDec.add(parteDecimaleBigDec);
		}
	}
	
	protected MixedDouble readFieldMixedDouble(java.io.InputStream in, int dimensioneInt,int dimensioneDec) throws IOException {
		String parteIntera = readFieldString(in, dimensioneInt);
		String parteDecimale = readFieldString(in, dimensioneDec);
		
		return new MixedDouble(parteIntera, parteDecimale);
	}
	
	protected void saltaFineRiga(java.io.InputStream in) throws IOException {
		in.skip(FINERECORD.length);
	}
	
	protected void scriviFineRiga(java.io.OutputStream out) throws IOException {
		out.write(FINERECORD);
	}
	
	protected byte[] normalizzaCampo(String campo, int dimensione) {
		String retvalue = "";
		
		if(campo == null) {
			campo = "";
		}		
		int length = dimensione - campo.length();
		StringBuffer buffer = new StringBuffer(campo);
		for(int i=0; i < length; i++) {
			appendToBuffer(buffer, ' ');
		}
		retvalue = buffer.toString();

		return retvalue.getBytes();
	}

	/**
	 * @deprecated  usate al posto normalizzaCampo che prende un BigInteger
	 * @param campo
	 * @param dimensione
	 * @return
	 */
	protected byte[] normalizzaCampo(int campo, int dimensione) {

		campo = Math.abs(campo);

		String retvalue = String.valueOf(campo);
		
		int length = dimensione - retvalue.length();
		StringBuffer buffer = new StringBuffer();
		for(int i=0; i < length; i++) {
			appendToBuffer(buffer, '0');
		}
		buffer.append(retvalue);
		retvalue = buffer.toString();
		
		return retvalue.getBytes();
	}

	protected byte[] normalizzaCampo(BigInteger campo, int dimensione) {

		if(campo.compareTo(BigInteger.valueOf(0))==-1) {
			// campo = campo.negate();
			BigInteger campoNew = campo.multiply(BigInteger.valueOf(-1));
			campo=campoNew;
		}

		String retvalue = campo.toString();
		
		int length = dimensione - retvalue.length();
		StringBuffer buffer = new StringBuffer();
		for(int i=0; i < length; i++) {
			appendToBuffer(buffer, '0');
		}
		buffer.append(retvalue);
		retvalue = buffer.toString();
		
		return retvalue.getBytes();
	}
	
	/**
	 * @deprecated  usate al posto normalizzaCampo che prende un BigDecimal
	 * @param campo
	 * @param dimensioneInt
	 * @param dimensioneDec
	 * @return
	 */
	protected byte[] normalizzaCampo(double campo, int dimensioneInt,int dimensioneDec) {
		
		campo = Math.abs(campo);

		String campoStringa = String.valueOf(campo);
		StringTokenizer tokenizer = new StringTokenizer(campoStringa, ".,");
		
		String parteIntera = tokenizer.nextToken();
		String parteDecimale = tokenizer.nextToken();

		StringBuffer buffer = null;
		int length = 0;			
		// Normalizzazione parteIntera	
		length = dimensioneInt - (parteIntera.length());
		buffer = new StringBuffer();
		for(int i=0; i < length; i++) {
			appendToBuffer(buffer, '0');
		}
		buffer.append(parteIntera);
		parteIntera = buffer.toString();

		// Normalizzazione parteDecimale	
		length = dimensioneDec - (parteDecimale.length());
		buffer = new StringBuffer();
		buffer.append(parteDecimale);
		for(int i=0; i < length; i++) {
			appendToBuffer(buffer, '0');
		}
		parteDecimale = buffer.toString();

		String retvalue = parteIntera + parteDecimale;
		
		return retvalue.getBytes();
	}

	protected byte[] normalizzaCampo(BigDecimal campo, int dimensioneInt,int dimensioneDec) {
		
		if(campo.compareTo(BigDecimal.valueOf(0)) < 0) {
			campo = campo.multiply(BigDecimal.valueOf(-1));
		}

		String campoStringa = campo.toString();

		StringTokenizer tokenizer = new StringTokenizer(campoStringa, ".,");
		
		String parteIntera = "";
		if (tokenizer.hasMoreTokens()) parteIntera = tokenizer.nextToken();

		String parteDecimale = "";
		if (tokenizer.hasMoreTokens()) parteDecimale = tokenizer.nextToken();

		StringBuffer buffer = null;
		int length = 0;			
		// Normalizzazione parteIntera	
		length = dimensioneInt - (parteIntera.length());
		buffer = new StringBuffer();
		for(int i=0; i < length; i++) {
			appendToBuffer(buffer, '0');
		}
		buffer.append(parteIntera);
		parteIntera = buffer.toString();

		// Normalizzazione parteDecimale	
		length = dimensioneDec - (parteDecimale.length());
		buffer = new StringBuffer();
		buffer.append(parteDecimale);
		for(int i=0; i < length; i++) {
			appendToBuffer(buffer, '0');
		}
		parteDecimale = buffer.toString();

		String retvalue = parteIntera + parteDecimale;
		
		return retvalue.getBytes();
	}

	protected byte[] normalizzaCampo(MixedDouble campo, int dimensioneInt, int dimensioneDec) {
		String parteIntera = new String(normalizzaCampo(campo.getIntValue(), dimensioneInt));
		String parteDecimale = String.valueOf(campo.getDecimalValue());

		StringBuffer buffer = new StringBuffer(parteIntera);
		buffer.append(parteDecimale);

		// Normalizzazione parteDecimale	
		int length = dimensioneDec - (parteDecimale.length());
		for(int i=0; i < length; i++) {
			appendToBuffer(buffer, '0');
		}

		return buffer.toString().getBytes();
	}	

	public Object clone() throws CloneNotSupportedException {
		return super.clone();
	}

	/**
	 * Metodo di utilit sviluppato per compilare sia con la 1.4 che con la 1.5
	 * @param buffer
	 * @param c
	 */
	public static void appendToBuffer(StringBuffer buffer, char c){
		try {
			buffer.append(c);
			throw new IOException("FAKE EXCEPTION");
		} catch (IOException e) {
		}
	}

}
