package it.finanze.dogane.accise.pdf;

import it.finanze.dogane.accise.model.Prospect;
import it.finanze.dogane.accise.ui.GrayGlassPane;
import it.finanze.dogane.accise.ui.PdfViewer;
import it.finanze.dogane.accise.ui.PrintDialog;
import it.finanze.dogane.accise.ui.quadri.DefaultEtichettaQuadro;
import it.finanze.dogane.accise.ui.quadri.GestoreQuadro;

import java.awt.Component;
import java.awt.Frame;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;

import javax.swing.JList;
import javax.swing.JOptionPane;
import javax.swing.RootPaneContainer;


public abstract class AbstractPdfProspectWriter implements Runnable {

	public static final int PRINT_ANYTHING = 0;
	public static final int PRINT_FRONTESPIZIO = 1;
	public static final int PRINT_QUADRO = 2;
	public static final int PRINT_ALL = 3;

	private Frame frame = null;
	private JList quadri = null;
	private Prospect prospect = null;
	private ByteArrayOutputStream os = new ByteArrayOutputStream();
	private PdfBuilder pdfBuilder = null;
	private int actionRequested = PRINT_ANYTHING;
	
	private Component realGlassPane = null;

	public void run() {
		if(frame == null || quadri == null || prospect == null) {
			throw new NullPointerException("Call print not run");
		}
		
		try {
			
			setGlassPanel();
						
			pdfBuilder = new PdfBuilder(os);

			if(actionRequested == PRINT_FRONTESPIZIO) {
				printFrontespizio(pdfBuilder);
			} else if(actionRequested == PRINT_ALL) {
				printFrontespizio(pdfBuilder);
				//pdfBuilder.newPage();
				//printQuadri();
				printQuadri(pdfBuilder);
			} else if(actionRequested == PRINT_QUADRO) {
				GestoreQuadro quadro = (GestoreQuadro)quadri.getModel().getElementAt(quadri.getSelectedIndex());
				//printQuadro(quadro);
				printQuadro(quadro,pdfBuilder);
			} else {
				throw new RuntimeException("Azione richiesta non gestita. Azione: " + actionRequested);
			}
			
			pdfBuilder.close();
			os.close();

			resetGlassPanel();

			showPDF();
//			writeToDisk();

		} catch (Exception e) {
			e.printStackTrace();
			JOptionPane.showMessageDialog(
				frame,
				"Errore durante l'operazione richiesta: \n" + e.getMessage(),
				"Errore",
				JOptionPane.ERROR_MESSAGE);
			
		}
	}

	private void setGlassPanel() {
		if(frame instanceof RootPaneContainer) {
			RootPaneContainer container = (RootPaneContainer)frame;
			realGlassPane = container.getGlassPane();
			GrayGlassPane glass = new GrayGlassPane();
			container.setGlassPane(glass);
			glass.setVisible(true);
		}
	}

	private void resetGlassPanel() {
		if(frame instanceof RootPaneContainer) {
			RootPaneContainer container = (RootPaneContainer)frame;
			container.setGlassPane(realGlassPane);
			if(realGlassPane != null) {
				realGlassPane.setVisible(false);
			}
		}
	}

	protected abstract void printFrontespizio(PdfBuilder pdfBuilder);
	protected abstract void printQuadro(GestoreQuadro quadro,PdfBuilder pdfBuilder);
	public abstract List getListaRecord(GestoreQuadro quadro);

	/**
	 * Utilizzare questo metodo per stampare
	 *
	 */
	public void print(JList quadri, Frame frame, Prospect prospect) {
		if(frame == null || quadri == null || prospect == null) {
			throw new NullPointerException();
		}
		
		this.frame = frame;
		this.quadri = quadri;
		this.prospect = prospect;
		
		if(quadri.getSelectedIndex() == -1) { // Non  selezionato nulla... va bu ci penso io
			quadri.setSelectedIndex(0); // Ma si almeno un valore c' sempre!
		}		
		
		PrintDialog dialog = new PrintDialog(frame);
		dialog.setVisible(true);

		this.actionRequested = dialog.getActionRequested();
		
		
		if(actionRequested == PRINT_QUADRO) {
			
			//se si seleziona un quadro che non contiene dati
			GestoreQuadro quadro = (GestoreQuadro)quadri.getModel().getElementAt(quadri.getSelectedIndex());
			
			boolean datiDaStampare = quadro.getProspect() != null  && getListaRecord(quadro).size() > 0;
			
			if(!datiDaStampare){
				
				JOptionPane.showMessageDialog(
								frame,
								"Il Quadro selezionato non contiene dati da stampare",
								"Errore",
								JOptionPane.ERROR_MESSAGE);
			
					
				dialog.setVisible(false);
				return;
			}
			
		}
		
			
		if(actionRequested != PRINT_ANYTHING) {
			Thread thread = new Thread(this);
			thread.start();
		}
		
	}


	private void printQuadri() {
		int size = quadri.getModel().getSize();
		for(int i = 0; i < size; i++) {
			GestoreQuadro quadro = (GestoreQuadro)quadri.getModel().getElementAt(i);
			printQuadro(quadro);
		}
	}
	
	private void printQuadri(PdfBuilder builder) {
		
	int size = quadri.getModel().getSize();
		for(int i = 0; i < size; i++) {
			GestoreQuadro quadro = (GestoreQuadro)quadri.getModel().getElementAt(i);
			List list = getListaRecord(quadro);
			
			if(list.size() > 0){//stampiamo solo i quadri valorizzati
				builder.newPage();//nuova pagina per ogni quadro
				printQuadro(quadro,builder);				
			}
		}
	}

	private void printQuadro(GestoreQuadro quadro) {
		DefaultEtichettaQuadro etichetta = (DefaultEtichettaQuadro)quadro;
			
		quadro.setProspect(prospect);
		if(!etichetta.isEmpty()) {
			if(etichetta.isComplex()) {
				List sottoQuadri = quadro.getSottoQuadri();
				Iterator it = sottoQuadri.iterator();
				while(it.hasNext()) {
					quadro.setProspect(prospect, it.next());
					quadro.printDialog(frame, pdfBuilder);
				}
			} else {
				quadro.printDialog(frame, pdfBuilder);
			}
		}
	}

	private void writeToDisk() throws IOException {
		
		PdfResourceAccess.saveAsFile(frame, os.toByteArray());
	}

	private void showPDF() {

		PdfViewer viewer = new PdfViewer(frame, os.toByteArray());
		viewer.setVisible(true);
	}

	/**
	 * @return
	 */
	protected Prospect getProspect() {
		return prospect;
	}

}
