package it.finanze.dogane.accise.pdf;

import it.finanze.dogane.accise.ui.TableUtil;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.Toolkit;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import javax.swing.JDialog;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.JTableHeader;
import javax.swing.table.TableModel;

import com.lowagie.text.Document;
import com.lowagie.text.DocumentException;
import com.lowagie.text.Element;
import com.lowagie.text.PageSize;
import com.lowagie.text.Rectangle;
import com.lowagie.text.pdf.BaseFont;
import com.lowagie.text.pdf.PdfContentByte;
import com.lowagie.text.pdf.PdfTemplate;
import com.lowagie.text.pdf.PdfWriter;


/**
 * Questa classe  capace di creare un PDF
 */
public class PdfBuilder {

	private OutputStream _os = null;
	private Document _document = null;
	private PdfWriter _writer = null;
	private PdfContentByte _cb = null;
	
	private float penY = TOP; // Tiene traccia di fin dove si  scritto.
	private float penX = MARGINE;
	private int pageNumber = 0;

	private PageEndWriterInterface pageEndWriterInterface = null;
	
	private List commandBlock = null;
	private float storeCommandPenY = TOP;
	private float tableHeaderSpaceRequired = -1;
	private List tableHeaderBlock = null;
	
	private boolean newPageCheckEnabled = true;

	public static final int MARGINE = 10;//30
	public static final int INTERLINEA =0;//2
	public static final int TOP = 10;//30
	public static final int BOTTOM = 10;//70
	
	public static final int ALIGN_RIGTH = Element.ALIGN_RIGHT;
	public static final int ALIGN_LEFT = Element.ALIGN_LEFT;
	public static final int ALIGN_CENTER = Element.ALIGN_CENTER;
	
	public int absoluteXDim = Toolkit.getDefaultToolkit().getScreenSize().width;
	public int absoluteYDim = Toolkit.getDefaultToolkit().getScreenSize().height;

	private static BaseFont FONT;
	static {
		try {
			FONT = BaseFont.createFont(BaseFont.HELVETICA, BaseFont.WINANSI, BaseFont.EMBEDDED);
		}
		catch(Exception e) {
		}
	}

	public PdfBuilder(OutputStream os) {
		_os = os;
		try {
			Rectangle rectangle = PageSize.A3;;
			//Rectangle rectangle = new Rectangle(800f, 600f);
			//Rectangle rectangle = new Rectangle(absoluteXDim, absoluteYDim);
			_document = new Document(rectangle);
			_document.setMargins(0f,0f,0f,0f);
			
			_writer = PdfWriter.getInstance(_document, os);				
			_writer.setPageEvent(new PageNumbers());		
			

			_document.open();
			
			
			_cb = _writer.getDirectContent();
			
			
		}
		catch(DocumentException e){
			e.printStackTrace();
			throw new RuntimeException(e.getMessage());
		}
	}

	/**
	 * Questo comando serve a scrivere una serie di label tutte in blocco in modo tale che
	 * non sia possibile che ci sia un'interruzione di pagina all'interno del blocco.
	 * I comandi utilizzabili con questo sono tutte le writeLabel, setXPen, moveXPen, e moveYPen
	 * Per quanto riguarda getXPen e getYPen si noti che le coordinate saranno riferite 
	 * all'interno del blocco e non alla posizione che il blocco assumer all'interno della pagina.
	 * Gli altri comandi se utilizzati fra una startCommandBlock e una endCommandBlock potrebbero causare risultati
	 * non prevedibili (Si generer quindi una RuntimeException).
	 * La scrittura del blocco fallir se l'ultimo comando impostato  una moveYPen con valore negativo
	 */
	public void startCommandBlock() {
		tableHeaderBlock = null;
		tableHeaderSpaceRequired = -1;
		commandBlock = new LinkedList();
		storeCommandPenY = penY;
		penX = MARGINE;
		penY = TOP;
	}

	public void endCommandBlock() {
		// System.out.println(tableHeaderSpaceRequired);
		if(tableHeaderSpaceRequired > -1) {
			throw new RuntimeException("metodo non richiamabile durante la definizione di un table header");
		}
		
		if(commandBlock != null) {
			Iterator it = commandBlock.iterator();
			commandBlock = null;
			float blockSpace = penY - TOP; // Si spera che l'ultimo comando non sia stato una moveY negativa!!!!!!
			penY = storeCommandPenY;
			if(blockSpace > getSpaceLeft()) {
				newPage();
			}
			
			while(it.hasNext()) {
				WriteCommand command = (WriteCommand)it.next();
				command.execute();
			}
		}
	}

	/**
	 * Questo metodo si comporta come startCommandBlock ma serve a definire un header per la successiva writeTable
	 * Non si pu utilizzare contemporaneamente al metodo startCommandBlock
	 */
	public void startDefineTableHeader() {
		startCommandBlock();
		tableHeaderBlock = null;
		tableHeaderSpaceRequired = 0;
	}

	public void endDefineTableHeader() {
		tableHeaderSpaceRequired = penY - TOP; // Si spera che l'ultimo comando non sia stato una moveY negativa!!!!!!
		penY = storeCommandPenY;
		tableHeaderBlock = commandBlock;
		commandBlock = null;
	}

	private void checkBlock() {
		if(commandBlock != null) {
			throw new RuntimeException("Comando non permesso durante l'esecuzione di un blocco");
		}
	}

	/**
	 * 
	 * @return true if the page was added, false if not
	 * @throws EsitiContabiliSystemException
	 */
	public boolean newPage() {
		checkBlock();
		boolean retvalue = false;
		if(pageEndWriterInterface != null) {
			pageEndWriterInterface.write(this);
			pageEndWriterInterface = null;
		}
		
		try {
			retvalue = _document.newPage();
		} catch (DocumentException e) {			
			e.printStackTrace();
		}
		pageNumber++;
		boolean isNewPageCheckEnabled = isNewPageCheckEnabled();
		setNewPageCheckEnabled(false);
		penY = _document.getPageSize().height() - 20;
		//writeRightLabel(String.valueOf(pageNumber), 10);
		setNewPageCheckEnabled(isNewPageCheckEnabled);
		penY = TOP;
		return retvalue;
	}

	public void setYPen(float y) {
		checkBlock();
		penY = y;
	}

	public float getYPen() {
		return penY;
	}

	public void setXPen(float x) {
		if(commandBlock != null) {
			WriteCommand command = new WriteCommand(WriteCommand.COMMAND_SETXPEN, x);
			commandBlock.add(command);
		}
		penX = x;
	}

	public float getXPen() {
		return penX;
	}

	public void moveYPen(float offy) {
		if(commandBlock != null) {
			WriteCommand command = new WriteCommand(WriteCommand.COMMAND_MOVEYPEN, offy);
			commandBlock.add(command);
		}
		penY += offy;
	}

	public void moveXPen(float offx) {
		if(commandBlock != null) {
			WriteCommand command = new WriteCommand(WriteCommand.COMMAND_MOVEXPEN, offx);
			commandBlock.add(command);
		}
		penX += offx;
	}


	/**
	 * Ritorna lo spazio rimasto prima di raggiungere la fine della pagina
	 * @return
	 */
	public float getSpaceLeft() {
		checkBlock();
		return _document.getPageSize().height() - penY - BOTTOM;
	}

	public void writeCenteredLabel(String label, int fontSize) {
		float x = _document.getPageSize().width() / 2;
		
		writeLabel(label, x, penY, fontSize, ALIGN_CENTER);
	}

	public void writeLeftLabel(String label, int fontSize) {

		writeLabel(label, penX, penY, fontSize, ALIGN_LEFT);
	}

	public void writeRightLabel(String label, int fontSize) {
		float x = _document.getPageSize().width() - MARGINE;

		writeLabel(label, x, penY, fontSize, ALIGN_RIGTH);
	}

	public void writeLabel(String label, int fontSize, int alignment) {
		if(alignment == ALIGN_LEFT) {
			writeLeftLabel(label, fontSize);
		} else if(alignment == ALIGN_CENTER) {
			writeCenteredLabel(label, fontSize);
		} else if(alignment == ALIGN_RIGTH) {
			writeRightLabel(label, fontSize);
		} else {
			throw new IllegalArgumentException("Allineamento non permesso. Valore : " + alignment);
		}
			
	}

	private void writeLabel(String label, float x, float y, int fontSize) {
			writeLabel(label, x, y, fontSize, ALIGN_LEFT);
	}

	private void writeLabel(String label, float x, float y, int fontSize, int alignment) {
		if(commandBlock != null) {
			WriteCommand command = new WriteCommand(label, fontSize, alignment);
			commandBlock.add(command);
		} else {

			if(newPageCheckEnabled && y > _document.getPageSize().height() - BOTTOM) {
				newPage();
				y = y - _document.getPageSize().height() + BOTTOM + TOP;
			}
			
			_cb.beginText();
			_cb.setFontAndSize(FONT, fontSize);
			_cb.showTextAligned(alignment, label, x, _document.getPageSize().height() - y - fontSize, 0);
	//		_cb.showTextAlignedKerned(alignment, label, x, _document.getPageSize().height() - y - fontSize, 0);
			_cb.endText();
		}
		penY = (int)y + fontSize + INTERLINEA;
	}

	/**
	 * Ritorna un oggetto awt in grado di disegnare sulla pagina
	 * Dopo averlo utilizzato richiamarne il metodo dispose.
	 * @return
	 */
	public Graphics2D getGraphics() {
		checkBlock();
		Graphics2D graphics = _cb.createGraphics(_document.getPageSize().width(), _document.getPageSize().height());
		return graphics;
	}

	/**
	 * Ritorna la dimensione di una singola pagina.
	 * Si pu utilizzare questo metodo per sapere quali
	 * sono le coordinate corrette per la visualizzazione.
	 * @return
	 */
	public Dimension getDimension() {
		checkBlock();
		Dimension d = new Dimension((int)_document.getPageSize().width(), (int)_document.getPageSize().height());
		return d;
	}

	/**
	 * Chiude l'editing del PDF producendolo interamente in output.
	 *
	 */
	public void close() {
		checkBlock();
		if(pageEndWriterInterface != null) {
			pageEndWriterInterface.write(this);
			pageEndWriterInterface = null;
		}
		
		_document.close();
	}
	
	/**
	 *  Stampa una tabella. ATTENZIONE questo metodo non deve essere richiamato dal DispatchThread della GUI
	 */
	public void writeTable(JDialog dialog, JTable table, JScrollPane scrollPane)
	{
		
		
		try {
			checkBlock();
		}
		catch(RuntimeException e) {
			if(tableHeaderSpaceRequired == -1) { // Allora stanno all'interno di un blocco
				throw e;
			} // Altrimenti hanno definito un table header
		}
		if(SwingUtilities.isEventDispatchThread()) {
			throw new RuntimeException("Non richiamare il metodo writeTable da EventDispatchThread");
		}
		if(tableHeaderSpaceRequired == -1) { 
			tableHeaderSpaceRequired = 0;
		}
		
		table = writePieceOfTable(dialog, table, scrollPane);
		
		while(table != null) {
			table = writePieceOfTable(dialog, table, scrollPane);
		}
		
		tableHeaderBlock = null;
		tableHeaderSpaceRequired = -1;
		
	}

	private JTable writePieceOfTable(JDialog dialog, JTable table, JScrollPane scrollPane) {

		Dimension pageDim = new Dimension((int)_document.getPageSize().width(), (int)_document.getPageSize().height());

		// Calcolo preventivo della scala
		scrollPane.setViewportView(table);
		table.validate();
		TableUtil.redimTable(table, scrollPane);
		
		synchDialog(dialog); 

		Dimension tableDimTmp = table.getSize();
//		double scaleX = (pageDim.getWidth() - ((MARGINE - 1) * 2)) / tableDimTmp.getWidth();
		double scaleX = pageDim.getWidth() / tableDimTmp.getWidth();


		int rowHeight = (int)((double)table.getRowHeight() * scaleX);
		int spaceLeft = (int)getSpaceLeft();
		spaceLeft -= tableHeaderSpaceRequired;
		int rowAllowed = spaceLeft / rowHeight;
		if(rowAllowed <= 0) {
			newPage();
			spaceLeft = (int)getSpaceLeft();
			spaceLeft -= tableHeaderSpaceRequired;
			rowAllowed = spaceLeft / rowHeight;
		}
		
		JTable tableToBePrinted = table;
		JTable tableNotPrinted = null;		

		if(rowAllowed < table.getRowCount() + 1) {
			
			PrintableTableModel model = (PrintableTableModel)table.getModel();
			PrintableTableModel modelToBePrinted = model.split(0, rowAllowed);
			PrintableTableModel modelNotPrinted = model.split(rowAllowed);
			
			tableToBePrinted = new JTable();
			tableToBePrinted.setModel((TableModel)modelToBePrinted);
			
			tableNotPrinted = new JTable();
			tableNotPrinted.setModel((TableModel)modelNotPrinted);
			
			if(tableNotPrinted.getRowCount() == 0) {
				tableNotPrinted = null;
			}
			
//			System.out.println("*********************");
//			System.out.println(tableToBePrinted.getModel().getRowCount());
//			System.out.println(tableNotPrinted.getModel().getRowCount());
//			System.out.println("*********************");
			
		}

		tableToBePrinted.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF);
		tableToBePrinted.getSelectionModel().clearSelection();
		
		

		scrollPane.setViewportView(tableToBePrinted);
		tableToBePrinted.validate();
		//TableUtil.redimTable(tableToBePrinted, scrollPane);
		TableUtil.redimTable(table, tableToBePrinted);
		//TableUtil.adjustColumnPreferredWidths(tableToBePrinted);
		
		synchDialog(dialog); 

		Dimension headerDim =tableToBePrinted.getTableHeader().getPreferredSize();
		Dimension tableDim = tableToBePrinted.getSize();
		scaleX = pageDim.getWidth() / tableDim.getWidth();
		// double scaleX = (pageDim.getWidth() - ((MARGINE - 1) * 2)) / tableDim.getWidth();

		JTableHeader tableHeader = tableToBePrinted.getTableHeader();
		

		// Stampo il TableHeader
		if(tableHeaderBlock != null) {
			Iterator it = tableHeaderBlock.iterator();
			
			while(it.hasNext()) {
				
				WriteCommand command = (WriteCommand)it.next();
				//command.setCFontSize(15);
				
				command.execute();
			}
		}

		// Stampo l'intestazione
		_cb.saveState();
		Graphics2D g2 = _cb.createGraphics((float)pageDim.width, (float)pageDim.height);
		
		

		Shape oldClip = g2.getClip();
		g2.setClip(null);
		//g2.translate(MARGINE, penY);
		g2.translate(0, penY);
		// g2.scale(scaleX, 1);
		g2.scale(scaleX, scaleX);
		tableHeader.print(g2);
		g2.scale(1, 1);
		penY += headerDim.height * scaleX;
		g2.setClip(oldClip);

		g2.dispose();
		_cb.restoreState();

		// Stampo la tabella
		_cb.saveState();
		g2 = _cb.createGraphics((float)pageDim.width, (float)pageDim.height);
		

		oldClip = g2.getClip();
		g2.setClip(null);
		//g2.translate(MARGINE, penY);
		g2.translate(0, penY);
		// g2.scale(scaleX, 1);
		g2.scale(scaleX, scaleX);
		tableToBePrinted.print(g2);
		g2.scale(1, 1);
		g2.setClip(oldClip);

		g2.dispose();
		_cb.restoreState();

		//PdfContentByte over = _writer.getDirectContent();
		
		 
		 
		 
		Rectangle rectangle = PageSize.A3;//
		//Rectangle rectangle = new Rectangle(absoluteXDim, absoluteYDim);

		PdfTemplate template = _cb.createTemplate(rectangle.width(), rectangle.height());		
		float w = (float)(tableToBePrinted.getSize().width * scaleX);
		float h = (float)(tableToBePrinted.getSize().height * scaleX);
		template.moveTo(0, rectangle.height() - penY - h);
		template.rectangle(0, rectangle.height() - penY - h, w, h);
		//template.lineTo(0, rectangle.height() - penY);
		template.stroke();
		
		
		// we add the template on different positions
		_cb.addTemplate(template, 0, 0);


		penY += tableToBePrinted.getSize().height * scaleX;

		return tableNotPrinted;
	}

	/**
	 * @param dialog
	 */
	private synchronized void synchDialog(JDialog dialog) {
		dialog.setVisible(false);
		Thread thread = new Thread(new ShowMan(dialog, true));
		thread.start();
		
		do {
			try {
				wait(20);
			} catch (InterruptedException e) {
				e.printStackTrace();
			}
		} while(dialog.isVisible() == false);
	}

	/**
	 * Rende il componente visibile o invisibile
	 */
	private class ShowMan implements Runnable {

		private boolean visible = false;
		private Component component = null;

		public ShowMan(Component c, boolean visible) {

			this.visible = visible;
			this.component = c;
		}

		public void run() {
			component.setVisible(visible);
		}

	}
	
	/**
	 * @param interf
	 */
	public void setPageEndWriterInterface(PageEndWriterInterface interf) {
		checkBlock();
		pageEndWriterInterface = interf;
	}

	// writeLabel, setXPen, moveXPen, moveYPen
	public class WriteCommand {
		private int commandID = COMMAND_WRITELABEL;
		private String cLabel = "";
		private float cValue = 0;
		private int   cFontSize = 0;
		private int   cAlignment = ALIGN_LEFT;
		
		public static final int COMMAND_WRITELABEL = 0;
		public static final int COMMAND_SETXPEN = 1;
		public static final int COMMAND_MOVEXPEN = 2;
		public static final int COMMAND_MOVEYPEN = 3;
		
		/**
		 * Costruttore per il comando di scrittura di una label
		 * @param label
		 * @param x
		 * @param y
		 * @param fontSize
		 * @param alignment
		 */
		public WriteCommand(String label, int fontSize, int alignment) {
			cLabel = label;
			cFontSize = fontSize;
			cAlignment = alignment;
		}

		/**
		 * il commandID pu assumere i valori COMMAND_SETXPEN, COMMAND_MOVEXPEN e COMMAND_MOVEYPEN
		 * @param commandID
		 * @param value
		 */
		public WriteCommand(int cmdID, float value) {
			if(cmdID == COMMAND_SETXPEN || cmdID == COMMAND_MOVEXPEN || cmdID == COMMAND_MOVEYPEN) {
				commandID = cmdID;
				cValue = value;
			} else {
				throw new IllegalArgumentException("Command ID non valido : " + cmdID);
			}
		}
		
		public void execute() {
			if(commandID == COMMAND_WRITELABEL) {
				writeLabel(cLabel, cFontSize, cAlignment);
			} else if(commandID == COMMAND_SETXPEN) {
				setXPen(cValue);
			} else if(commandID == COMMAND_MOVEXPEN) {
				moveXPen(cValue);
			} else if(commandID == COMMAND_MOVEYPEN) {
				moveYPen(cValue);
			}
		}

		/**
		 * @return
		 */
		public int getCFontSize() {
			return cFontSize;
		}

		/**
		 * @param i
		 */
		public void setCFontSize(int i) {
			cFontSize = i;
		}

	}
	/**
	 * @return
	 */
	public boolean isNewPageCheckEnabled() {
		return newPageCheckEnabled;
	}

	/**
	 * @param b
	 */
	public void setNewPageCheckEnabled(boolean b) {
		newPageCheckEnabled = b;
	}

	/**
	 * @return
	 */
	public Document getDocument() {
		return _document;
	}

}
