package it.finanze.dogane.accise.ui;


import it.finanze.dogane.accise.util.StringUtil;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.LayoutManager;
import java.awt.Rectangle;

import javax.swing.JScrollPane;
import javax.swing.JTable;

/**
 *    Ridimensiona un componente considerando la sua taglia assoluta come relativa ad uno schermo 800x600
 *    considerando quindi la grandezza reale dello schermo ridimensiona il componente (e lo sposta)
 * 
 * 	  NB. Per gestire il non ridimensionamento ulteriore si utilizza la proprieta name del componente impostado
 *        la parte finale del nome con la stringa _REDIM_DONE
 */
public class FullScreenLayoutManager implements LayoutManager {

	private static FullScreenLayoutManager instance = new FullScreenLayoutManager();
	
	private FullScreenLayoutManager() {}
	
	// private Set components = new HashSet(); // Si  eliminato l'insieme per evitare problemi di memory leak
	
	public static FullScreenLayoutManager getInstance() {
		return instance;
	}
	
	public void addLayoutComponent(String string, Component component) {
	}

	public void removeLayoutComponent(Component component) {
	}

	public void layoutContainer(Container container) {
		redim(container);
		
		Component[] cs = container.getComponents();
		for(int i=0; i < cs.length; i++) {
			if(cs[i] instanceof Container) {
				layoutContainer((Container)cs[i]);
			} else {
				redim(cs[i]);
			}
		}
		container.repaint();
	}


	public Dimension minimumLayoutSize(Container container) {
		redim(container);
		return container.getPreferredSize();
	}

	public Dimension preferredLayoutSize(Container container) {
		return redim(container);
	}


	private Dimension redim(Component component) {
		Dimension retvalue = null;
		String name = StringUtil.denullify(component.getName());
		Rectangle bounds = component.getBounds();
		if(!name.endsWith("_REDIM_DONE")) {
//		if(!components.contains(component)) {
			
			bounds.x = RealDim.getAbsoluteXValue(bounds.x);
			bounds.y = RealDim.getAbsoluteYValue(bounds.y);
			bounds.width = RealDim.getAbsoluteXValue(bounds.width);
			bounds.height = RealDim.getAbsoluteYValue(bounds.height);
			component.setBounds(bounds);

			if(component instanceof JScrollPane) {
				JScrollPane pane = (JScrollPane)component;
				Component view = pane.getViewport().getView();
				if(view instanceof JTable) {
					TableUtil.redimTable((JTable)view, pane);
				}
			}

//			components.add(component);
			component.setName(name + "_REDIM_DONE");
		}
		retvalue = new Dimension(bounds.width, bounds.height);
		return retvalue;
	}
}
