package it.finanze.dogane.accise.ui;

import it.finanze.dogane.accise.model.Interchange;
import it.finanze.dogane.accise.ui.validation.BigIntegerDocument;
import it.finanze.dogane.accise.ui.validation.StringDocument;

import java.awt.Color;
import java.awt.Font;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.math.BigInteger;
import java.util.Calendar;
import java.util.Locale;

import javax.swing.BorderFactory;
import javax.swing.JDialog;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.SwingConstants;
import javax.swing.border.Border;
import javax.swing.border.TitledBorder;
import javax.swing.JOptionPane;

public class InterchangeDialog extends JDialog implements ActionListener {

	private javax.swing.JPanel jContentPane = null;
	private javax.swing.JPanel jPanelDatiIterchange = null;
	private javax.swing.JLabel jLabelTitolo = null;
	private javax.swing.JLabel jLabelUtenteAbilitato = null;
	private javax.swing.JTextField jTextFieldUtenteAbilitato = null;
	private javax.swing.JLabel jLabelProgressivoSede = null;
	private javax.swing.JTextField jTextFieldProgressivoSede = null;
	private javax.swing.JLabel jLabelCodFiscPartitaIva1 = null;
	private javax.swing.JTextField jTextFieldCodFiscPartitaIva = null;
	private javax.swing.JLabel jLabelDataInterchange = null;
	private javax.swing.JTextField jTextFieldGGInterchange = null;
	private javax.swing.JTextField jTextFieldMMInterchange = null;
	private javax.swing.JLabel jLabelProgInterchange = null;
	private javax.swing.JTextField jTextFieldProgInterchange = null;
	private javax.swing.JLabel jLabelNomeFlusso = null;
	private javax.swing.JTextField jTextFieldNomeFlusso = null;
	private javax.swing.JButton jButtonChiudi = null;
	private javax.swing.JTextField jTextFieldFileInterchange = null;
	private JLabel jLabelListaFile = null;
	private javax.swing.JButton jButtonAddFile = null;
	private javax.swing.JButton jButtonEliminaFile = null;
	private javax.swing.JLabel jLabelFileInterc = null;
	private javax.swing.JButton jButtonCreaInterchange = null;

	/**
	 * This is the default constructor
	 */
	public InterchangeDialog(Frame frame,String title,boolean isModal) {
		super(frame, title, isModal);
		initialize();
	}

	protected void loadModelFromUI(Interchange model) {
		// carico il modello con dati dalla GUI
		if (!jTextFieldUtenteAbilitato.getText().trim().equals(""))
			model.setCodiceUtenteAbilitato(
				this.jTextFieldUtenteAbilitato.getText().trim());
		if (!jTextFieldProgressivoSede.getText().trim().equals(""))
			model.setProgressivoSede(
				new BigInteger(
					this.jTextFieldProgressivoSede.getText().trim()));
		if (!jTextFieldMMInterchange.getText().trim().equals(""))
			model.setMeseInoltro(
				new BigInteger(this.jTextFieldMMInterchange.getText()));
		if (!jTextFieldGGInterchange.getText().trim().equals(""))
			model.setGiornoInoltro(
				new BigInteger(this.jTextFieldGGInterchange.getText()));
		if (!jTextFieldCodFiscPartitaIva.getText().trim().equals(""))
			model.setCodiceFiscalePartitaIva(
				this.jTextFieldCodFiscPartitaIva.getText().trim());
		if (!jTextFieldProgInterchange.getText().trim().equals(""))
			model.setProgressivoInterchange(
				this.jTextFieldProgInterchange.getText().trim());
		if (!jTextFieldNomeFlusso.getText().trim().equals(""))
			model.setNomeFlusso(this.jTextFieldNomeFlusso.getText().trim());
		if (!jTextFieldFileInterchange.getText().trim().equals(""))
			model.setFileNameInterchange(
				this.jTextFieldFileInterchange.getText().trim());
	}

	protected void loadUIFromModel(Interchange model) {
		// carico la GUI con i dati presenti nel model
		if (!model.getCodiceUtenteAbilitato().equals(""))
			this.jTextFieldUtenteAbilitato.setText(
				model.getCodiceUtenteAbilitato());
		if (!model.getProgressivoSede().equals(BigInteger.valueOf(0)))
			jTextFieldProgressivoSede.setText(
				model.getProgressivoSede().toString());
		if (!model.getMeseInoltro().equals(BigInteger.valueOf(0)))
			jTextFieldMMInterchange.setText(getMese(model.getMeseInoltro()));
		if (!model.getGiornoInoltro().equals(BigInteger.valueOf(0)))
			jTextFieldGGInterchange.setText(
				getGiorno(model.getGiornoInoltro()));
		if (!model.getCodiceFiscalePartitaIva().equals(""))
			this.jTextFieldCodFiscPartitaIva.setText(
				model.getCodiceFiscalePartitaIva());
		if (!model.getProgressivoInterchange().equals(""))
			this.jTextFieldProgInterchange.setText(
				model.getProgressivoInterchange());
		if (!model.getNomeFlusso().equals(""))
			this.jTextFieldNomeFlusso.setText(model.getNomeFlusso());
		if (!model.getFileNameInterchange().equals(""))
			this.jTextFieldFileInterchange.setText(
				model.getFileNameInterchange());
	}

	protected String getMese(BigInteger meseInt) {
		// mette uno zero davanti al int del mese se compreso tra 1 e 9
		String mese = meseInt.toString();
		if (meseInt.compareTo(BigInteger.valueOf(10)) == -1) {
			mese = "0" + mese;
		}
		return mese;
	}
	protected String getGiorno(BigInteger giornoInt) {
		// mette uno zero davanti al int del giorno se compreso tra 1 e 9
		String giorno = giornoInt.toString();
		if (giornoInt.compareTo(BigInteger.valueOf(10)) == -1) {
			giorno = "0" + giorno;
		}
		return giorno;
	}

	protected boolean checkParametri() {
		boolean isOk=false;
		String msg = "";
		if (jTextFieldUtenteAbilitato.getText().equals("")
			|| jTextFieldUtenteAbilitato.getText().length()
				!= Interchange.DIM_CODICE_UTENTE_ABILITATO) {
			msg =
				"Iserire il codice identificativo dell'utente abilitato (4 caratteri)";
		} else if (
			jTextFieldProgressivoSede.getText().equals("")
				|| jTextFieldProgressivoSede.getText().length()
					!= Interchange.DIM_PROGRESSIVO_SEDE) {
			msg =
				"Iserire il progressivo sede utente autorizzato (3 caratteri)";
		} else if (
			jTextFieldCodFiscPartitaIva.getText().equals("")
				|| !(jTextFieldCodFiscPartitaIva.getText().length()
					== Interchange.DIM_CODICE_FISCALE_PARTITA_IVA
					|| jTextFieldCodFiscPartitaIva.getText().length() == 11)) {
			msg = "Iserire il codice fiscale o la Partita IVA del richiedente";
		} else if (
			jTextFieldProgInterchange.getText().equals("")
				|| jTextFieldProgInterchange.getText().length()
					!= Interchange.DIM_PROGRESSIVO_INTERCHANGE) {
			msg = "Inserire il progressivo dell'interchange (2 caratteri)";
		} else if (jTextFieldFileInterchange.getText().equals("")) {
			msg = "Non  stato scelto il file da inserire nell'interchange";
		}

		if (!msg.equals("")) {
			JOptionPane.showMessageDialog(
				this,
				msg,
				"Errore",
				JOptionPane.ERROR_MESSAGE);
			isOk=false;
		} else {
			isOk=true;
		}
		return isOk;
	}
	
	//	  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	//	  ///////     GESTIONE EVENTI                     /////////////////////////////////////////////////////////////////////////////////////////////////
	//	  /////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
	public void actionPerformed(ActionEvent event) {
		//empty....
	}

	protected void doClose() {
		this.setVisible(false);
	}

	/**
	 * This method initializes this
	 * 
	 * @return void
	 */
	private void initialize() {
//		this.setSize(792, 376);
		this.setSize(RealDim.getAbsoluteXValue(792), RealDim.getAbsoluteYValue(376));
		this.setDefaultCloseOperation(
			javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
		this.setContentPane(getJContentPane());

		int xDim = RealDim.getAbsoluteXDim();
		int yDim = RealDim.getAbsoluteYDim();
		int width = getSize().width;
		int height = getSize().height;
		this.setLocation((xDim - width) / 2, (yDim - height) / 2);

		this.setResizable(false);
		this.setModal(true);

		jTextFieldGGInterchange.setDocument(new BigIntegerDocument(Interchange.DIM_GIORNO_INOLTRO));
		jTextFieldMMInterchange.setDocument(new BigIntegerDocument(Interchange.DIM_MESE_INOLTRO));
		jTextFieldProgressivoSede.setDocument(new BigIntegerDocument(Interchange.DIM_PROGRESSIVO_SEDE));
		jTextFieldCodFiscPartitaIva.setDocument(new StringDocument(Interchange.DIM_CODICE_FISCALE_PARTITA_IVA));
		jTextFieldUtenteAbilitato.setDocument(new StringDocument(Interchange.DIM_CODICE_UTENTE_ABILITATO));
		jTextFieldProgInterchange.setDocument(new StringDocument(Interchange.DIM_PROGRESSIVO_INTERCHANGE));
	}
	/**
	 * This method initializes jContentPane
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJContentPane() {
		if (jContentPane == null) {
			jContentPane = new javax.swing.JPanel();
			jContentPane.setLayout(null);
			jContentPane.setLayout(FullScreenLayoutManager.getInstance());
			jContentPane.add(getJPanelDatiIterchange(), null);
		}
		return jContentPane;
	}
	/**
	 * This method initializes jPanelDatiIterchange
	 * 
	 * @return javax.swing.JPanel
	 */
	private javax.swing.JPanel getJPanelDatiIterchange() {
		if (jPanelDatiIterchange == null) {
			jPanelDatiIterchange = new javax.swing.JPanel();
			jPanelDatiIterchange.setLayout(null);
			jPanelDatiIterchange.setLayout(FullScreenLayoutManager.getInstance());
			jPanelDatiIterchange.add(getJLabelTitolo(), null);
			jPanelDatiIterchange.add(getJLabelUtenteAbilitato(), null);
			jPanelDatiIterchange.add(getJTextFieldUtenteAbilitato(), null);
			jPanelDatiIterchange.add(getJLabelProgressivoSede(), null);
			jPanelDatiIterchange.add(getJTextFieldProgressivoSede(), null);
			jPanelDatiIterchange.add(getJLabelCodFiscPartitaIva1(), null);
			jPanelDatiIterchange.add(getJTextFieldCodFiscPartitaIva(), null);
			jPanelDatiIterchange.add(getJLabelDataInterchange(), null);
			jPanelDatiIterchange.add(getJTextFieldGGInterchange(), null);
			jPanelDatiIterchange.add(getJTextFieldMMInterchange(), null);
			jPanelDatiIterchange.add(getJLabelProgInterchange(), null);
			jPanelDatiIterchange.add(getJTextFieldProgInterchange(), null);
			jPanelDatiIterchange.add(getJLabelNomeFlusso(), null);
			jPanelDatiIterchange.add(getJTextFieldNomeFlusso(), null);
			jPanelDatiIterchange.add(getJTextFieldFileInterchange(), null);
			jPanelDatiIterchange.add(getJLabelFileInterc(), null);
			jPanelDatiIterchange.add(getJButtonAddFile(), null);
			jPanelDatiIterchange.add(getJButtonEliminaFile(), null);
			jPanelDatiIterchange.add(getJButtonChiudi(), null);
			jPanelDatiIterchange.add(getJButtonCreaInterchange(), null);
			jPanelDatiIterchange.setBounds(12, 14, 758, 314);
			jPanelDatiIterchange.setBackground(
				new java.awt.Color(225, 225, 225));
		}
		return jPanelDatiIterchange;
	}
	/**
	 * This method initializes jLabelTitolo
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelTitolo() {
		if (jLabelTitolo == null) {
			jLabelTitolo = new javax.swing.JLabel();
			jLabelTitolo.setBounds(170, 7, 450, 31);
			jLabelTitolo.setText(
				"DATI PER LA COMPOSIZIONE DEL RECORD DI TESTATA DEL'INTERCHANGE");
			jLabelTitolo.setHorizontalAlignment(
				javax.swing.SwingConstants.CENTER);
			jLabelTitolo.setHorizontalTextPosition(
				javax.swing.SwingConstants.CENTER);
			jLabelTitolo.setFocusable(false);
		}
		return jLabelTitolo;
	}
	/**
	 * This method initializes jLabelUtenteAbilitato
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelUtenteAbilitato() {
		if (jLabelUtenteAbilitato == null) {
			jLabelUtenteAbilitato = new javax.swing.JLabel();
			jLabelUtenteAbilitato.setBounds(14, 45, 241, 21);
			jLabelUtenteAbilitato.setText("Codice utente abilitato:");
			jLabelUtenteAbilitato.setHorizontalAlignment(
				javax.swing.SwingConstants.LEFT);
			jLabelUtenteAbilitato.setHorizontalTextPosition(
				javax.swing.SwingConstants.LEFT);
			jLabelUtenteAbilitato.setAlignmentX(0.0F);
			jLabelUtenteAbilitato.setFocusable(false);
		}
		return jLabelUtenteAbilitato;
	}
	/**
	 * This method initializes jTextFieldUtenteAbilitato
	 * 
	 * @return javax.swing.JTextField
	 */
	protected javax.swing.JTextField getJTextFieldUtenteAbilitato() {
		if (jTextFieldUtenteAbilitato == null) {
			jTextFieldUtenteAbilitato = new javax.swing.JTextField();
			jTextFieldUtenteAbilitato.setBounds(265, 45, 250, 21);
		}
		return jTextFieldUtenteAbilitato;
	}
	/**
	 * This method initializes jLabelProgressivoSede
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelProgressivoSede() {
		if (jLabelProgressivoSede == null) {
			jLabelProgressivoSede = new javax.swing.JLabel();
			jLabelProgressivoSede.setBounds(14, 72, 241, 21);
			jLabelProgressivoSede.setText("Progressivo sede:");
			jLabelProgressivoSede.setHorizontalAlignment(
				javax.swing.SwingConstants.LEFT);
			jLabelProgressivoSede.setHorizontalTextPosition(
				javax.swing.SwingConstants.LEFT);
			jLabelProgressivoSede.setFocusable(false);
		}
		return jLabelProgressivoSede;
	}
	/**
	 * This method initializes jTextFieldProgressivoSede
	 * 
	 * @return javax.swing.JTextField
	 */
	private javax.swing.JTextField getJTextFieldProgressivoSede() {
		if (jTextFieldProgressivoSede == null) {
			jTextFieldProgressivoSede = new javax.swing.JTextField();
			jTextFieldProgressivoSede.setBounds(265, 72, 251, 21);
		}
		return jTextFieldProgressivoSede;
	}
	/**
	 * This method initializes jLabelCodFiscPartitaIva1
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelCodFiscPartitaIva1() {
		if (jLabelCodFiscPartitaIva1 == null) {
			jLabelCodFiscPartitaIva1 = new javax.swing.JLabel();
			jLabelCodFiscPartitaIva1.setBounds(14, 98, 241, 21);
			jLabelCodFiscPartitaIva1.setText(
				"Codce Fiscale o Partita Iva del richiedente:");
			jLabelCodFiscPartitaIva1.setFont(
				new java.awt.Font("Dialog", java.awt.Font.BOLD, 12));
			jLabelCodFiscPartitaIva1.setHorizontalAlignment(
				javax.swing.SwingConstants.LEFT);
			jLabelCodFiscPartitaIva1.setHorizontalTextPosition(
				javax.swing.SwingConstants.LEFT);
			jLabelCodFiscPartitaIva1.setVerticalAlignment(
				javax.swing.SwingConstants.CENTER);
			jLabelCodFiscPartitaIva1.setVerticalTextPosition(
				javax.swing.SwingConstants.CENTER);
			jLabelCodFiscPartitaIva1.setFocusable(false);
		}
		return jLabelCodFiscPartitaIva1;
	}
	/**
	 * This method initializes jTextFieldCodFiscPartitaIva
	 * 
	 * @return javax.swing.JTextField
	 */
	private javax.swing.JTextField getJTextFieldCodFiscPartitaIva() {
		if (jTextFieldCodFiscPartitaIva == null) {
			jTextFieldCodFiscPartitaIva = new javax.swing.JTextField();
			jTextFieldCodFiscPartitaIva.setBounds(265, 98, 251, 21);
		}
		return jTextFieldCodFiscPartitaIva;
	}
	/**
	 * This method initializes jLabelDataInterchange
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelDataInterchange() {
		if (jLabelDataInterchange == null) {
			jLabelDataInterchange = new javax.swing.JLabel();
			jLabelDataInterchange.setBounds(13, 124, 241, 21);
			jLabelDataInterchange.setText(
				"Data di inoltro dell'Interchange (gg/mm) :");
			jLabelDataInterchange.setFocusable(false);
		}
		return jLabelDataInterchange;
	}
	/**
	 * This method initializes jTextFieldGGInterchange
	 * 
	 * @return javax.swing.JTextField
	 */
	protected javax.swing.JTextField getJTextFieldGGInterchange() {
		if (jTextFieldGGInterchange == null) {
			jTextFieldGGInterchange = new javax.swing.JTextField();
			jTextFieldGGInterchange.setBounds(265, 124, 30, 21);
		}
		return jTextFieldGGInterchange;
	}
	/**
	 * This method initializes jTextFieldMMInterchange
	 * 
	 * @return javax.swing.JTextField
	 */
	protected javax.swing.JTextField getJTextFieldMMInterchange() {
		if (jTextFieldMMInterchange == null) {
			jTextFieldMMInterchange = new javax.swing.JTextField();
			jTextFieldMMInterchange.setBounds(300, 124, 30, 21);
		}
		return jTextFieldMMInterchange;
	}
	/**
	 * This method initializes jLabelProgInterchange
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelProgInterchange() {
		if (jLabelProgInterchange == null) {
			jLabelProgInterchange = new javax.swing.JLabel();
			jLabelProgInterchange.setBounds(13, 150, 241, 21);
			jLabelProgInterchange.setText("Progressivo Interchange");
			jLabelProgInterchange.setFocusable(false);
		}
		return jLabelProgInterchange;
	}
	/**
	 * This method initializes jTextFieldProgInterchange
	 * 
	 * @return javax.swing.JTextField
	 */
	protected javax.swing.JTextField getJTextFieldProgInterchange() {
		if (jTextFieldProgInterchange == null) {
			jTextFieldProgInterchange = new javax.swing.JTextField();
			jTextFieldProgInterchange.setBounds(265, 150, 30, 21);
		}
		return jTextFieldProgInterchange;
	}
	/**
	 * This method initializes jLabelNomeFlusso
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelNomeFlusso() {
		if (jLabelNomeFlusso == null) {
			jLabelNomeFlusso = new javax.swing.JLabel();
			jLabelNomeFlusso.setBounds(13, 206, 241, 21);
			jLabelNomeFlusso.setText("Nome flusso");
			jLabelNomeFlusso.setFocusable(false);
		}
		return jLabelNomeFlusso;
	}
	/**
	 * This method initializes jTextFieldNomeFlusso
	 * 
	 * @return javax.swing.JTextField
	 */
	protected javax.swing.JTextField getJTextFieldNomeFlusso() {
		if (jTextFieldNomeFlusso == null) {
			jTextFieldNomeFlusso = new javax.swing.JTextField();
			jTextFieldNomeFlusso.setBounds(265, 206, 251, 21);
			jTextFieldNomeFlusso.setEditable(false);
			jTextFieldNomeFlusso.setEnabled(true);
			jTextFieldNomeFlusso.setFocusable(true);
			jTextFieldNomeFlusso.setBackground(Color.WHITE);
		}
		return jTextFieldNomeFlusso;
	}
	/**
	 * This method initializes jButtonChiudi
	 * 
	 * @return javax.swing.JButton
	 */
	protected javax.swing.JButton getJButtonChiudi() {
		if (jButtonChiudi == null) {
			jButtonChiudi = new javax.swing.JButton();
			jButtonChiudi.setBounds(621, 255, 123, 28);
			jButtonChiudi.setText("Chiudi");
			jButtonChiudi.addActionListener(this);
		}
		return jButtonChiudi;
	}
	/**
	 * This method initializes getJTextFieldListaFile
	 * 
	 * @return javax.swing.JTextField
	 */
	protected javax.swing.JTextField getJTextFieldFileInterchange() {
		if (jTextFieldFileInterchange == null) {
			jTextFieldFileInterchange = new javax.swing.JTextField();
			jTextFieldFileInterchange.setBounds(265, 178, 251, 21);
			jTextFieldFileInterchange.setEditable(false);
			jTextFieldFileInterchange.setEnabled(true);
			jTextFieldFileInterchange.setFocusable(true);
			jTextFieldFileInterchange.setBackground(Color.WHITE);
		}
		return jTextFieldFileInterchange;
	}
	/**
	 * This method initializes jButtonAddFile
	 * 
	 * @return javax.swing.JButton
	 */
	protected javax.swing.JButton getJButtonAddFile() {
		if (jButtonAddFile == null) {
			jButtonAddFile = new javax.swing.JButton();
			jButtonAddFile.setBounds(526, 178, 106, 21);
			jButtonAddFile.setText("Aggiungi file");
			jButtonAddFile.setHorizontalTextPosition(
				javax.swing.SwingConstants.CENTER);
			jButtonAddFile.addActionListener(this);
		}
		return jButtonAddFile;
	}
	/**
	 * This method initializes jButtonEliminaFile
	 * 
	 * @return javax.swing.JButton
	 */
	protected javax.swing.JButton getJButtonEliminaFile() {
		if (jButtonEliminaFile == null) {
			jButtonEliminaFile = new javax.swing.JButton();
			jButtonEliminaFile.setBounds(638, 178, 106, 21);
			jButtonEliminaFile.setText("Elimina file");
			jButtonEliminaFile.setHorizontalTextPosition(
				javax.swing.SwingConstants.CENTER);
			jButtonEliminaFile.addActionListener(this);
		}
		return jButtonEliminaFile;
	}
	/**
	 * This method initializes jLabelFileInterc
	 * 
	 * @return javax.swing.JLabel
	 */
	private javax.swing.JLabel getJLabelFileInterc() {
		if (jLabelFileInterc == null) {
			jLabelFileInterc = new javax.swing.JLabel();
			jLabelFileInterc.setBounds(13, 178, 241, 21);
			jLabelFileInterc.setText("File da inserire nell'Interchange");
			jLabelFileInterc.setFocusable(false);
		}
		return jLabelFileInterc;
	}
	/**
	 * This method initializes jButtonCreaInterchange
	 * 
	 * @return javax.swing.JButton
	 */
	protected javax.swing.JButton getJButtonCreaInterchange() {
		if (jButtonCreaInterchange == null) {
			jButtonCreaInterchange = new javax.swing.JButton();
			jButtonCreaInterchange.setBounds(526, 206, 218, 21);
			jButtonCreaInterchange.setText("Crea Interchange");
			jButtonCreaInterchange.addActionListener(this);
		}
		return jButtonCreaInterchange;
	}
}  //  @jve:visual-info  decl-index=0 visual-constraint="10,10"
