package it.finanze.dogane.accise.ui;

import java.awt.Component;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.FontMetrics;

import javax.swing.JLabel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

public class TableUtil implements Runnable {

	private JTable jTable = null;
	private JScrollPane pane = null;

	private TableUtil() {}

	/**
	 * Ridimensiona le colonne di una tabella in modo da visualizzare esattamente quello che c' dentro
	 * pane pu essere nullo la chiamata viene effettuata dopo l'esaurimento di tutti gli eventi del dispatch thread
	 * @param jTable
	 * @param pane
	 */
	public static void redimTableLater(JTable jTable, JScrollPane pane) {
		TableUtil tu = new TableUtil();
		tu.jTable = jTable;
		tu.pane = pane;
		
		SwingUtilities.invokeLater(tu);
	}

	/**
	 * Ridimensiona le colonne di una tabella in modo da visualizzare esattamente quello che c' dentro
	 * pane pu essere nullo
	 * @param jTable
	 * @param pane
	 */
	public static void redimTable(JTable jTable, JScrollPane pane) {
		DefaultTableCellRenderer render = new DefaultTableCellRenderer();
		int colCount = jTable.getColumnCount();
		int rowCount = jTable.getRowCount();
		
		int[] maxWidths = new int[colCount];
		int totWidth = 0;
		
		for(int col=0; col < colCount; col++) {
			int maxWidth = 0;
			Font font = null;
			FontMetrics fontMetrics = null;
			for(int row=0; row < rowCount; row++) {
				Object obj = jTable.getModel().getValueAt(row, col);
				if(obj != null) {
					String value = obj.toString();
					
					JLabel label = (JLabel)render.getTableCellRendererComponent(jTable, value, false, false, row, col);
					font = label.getFont();
					fontMetrics = label.getFontMetrics(font);
					/*font = new Font("SansSerif", Font.BOLD | Font.ITALIC, 30);
					fontMetrics =label.getFontMetrics(font);
					int width =50;*/
					
					int width = fontMetrics.stringWidth(value);
		
					width += label.getInsets().left + label.getInsets().right + 2; // + 2 non so ma funziona!
					if(width > maxWidth) {
						maxWidth = width;
					}
				}
			}
			
			// Vediamo quando  grande l'etichetta + o -
			TableColumn column = jTable.getColumnModel().getColumn(col);
			String headValue = column.getHeaderValue().toString();
			// Uso il font dell'ultima cella (anche se questa cosa non  precisa!)
			if(fontMetrics != null) {
				int width = fontMetrics.stringWidth(headValue) + 4; //Aggiungo 4 come forfait
				if(width > maxWidth) {
					maxWidth = width;
				}
			}
			
			maxWidths[col] = maxWidth;
			totWidth += maxWidth;
		}
		
		if(pane != null) {
			int currentWidth = pane.getSize().width /* - pane.getInsets().left - pane.getInsets().right */;  
			if(totWidth < currentWidth) {
				int difference = (currentWidth - totWidth) / colCount;
				for(int i=0; i<colCount; i++) {
					maxWidths[i] += difference;
				}
			}
		}
		// And now lets redim!
		for(int i=0; i<colCount; i++) {
//			jTable.getColumnModel().getColumn(col).setMinWidth(maxWidth);
			jTable.getColumnModel().getColumn(i).setPreferredWidth(maxWidths[i]);
		}
		
		
	}


	public static void redimTableMinimumSize(JTable jTable, JScrollPane pane) {
		DefaultTableCellRenderer render = new DefaultTableCellRenderer();
		int colCount = jTable.getColumnCount();
		int rowCount = jTable.getRowCount();
		
		int[] maxWidths = new int[colCount];
		int totWidth = 0;
		
		for(int col=0; col < colCount; col++) {
			int maxWidth = 0;
			Font font = null;
			FontMetrics fontMetrics = null;
			for(int row=0; row < rowCount; row++) {
				Object obj = jTable.getModel().getValueAt(row, col);
				if(obj != null) {
					String value = obj.toString();
					JLabel label = (JLabel)render.getTableCellRendererComponent(jTable, value, false, false, row, col);
					font = label.getFont();
					fontMetrics = label.getFontMetrics(font);
					int width = fontMetrics.stringWidth(value);
		
					width += label.getInsets().left + label.getInsets().right + 2; // + 2 non so ma funziona!
					if(width > maxWidth) {
						maxWidth = width;
					}
				}
			}
			
			// Vediamo quando  grande l'etichetta + o -
			TableColumn column = jTable.getColumnModel().getColumn(col);
			String headValue = column.getHeaderValue().toString();
			// Uso il font dell'ultima cella (anche se questa cosa non  precisa!)
			if(fontMetrics != null) {
				int width = fontMetrics.stringWidth(headValue) + 4; //Aggiungo 4 come forfait
				if(width > maxWidth) {
					maxWidth = width;
				}
			}
			
			maxWidths[col] = maxWidth;
			totWidth += maxWidth;
		}
		
		// And now lets redim!
		for(int i=0; i<colCount; i++) {
//			jTable.getColumnModel().getColumn(col).setMinWidth(maxWidth);
			jTable.getColumnModel().getColumn(i).setPreferredWidth(maxWidths[i]);
		}
		
		
		
	}


	public static void redimTable(JTable srcTable, JTable destTable) {
		int colCount = srcTable.getColumnCount();
		
		for(int i=0; i<colCount; i++) {
			int width = srcTable.getColumnModel().getColumn(i).getPreferredWidth();
			destTable.getColumnModel().getColumn(i).setPreferredWidth(width);//width
			
		}		
	
		
	}
	
	
		
	public static void adjustColumnPreferredWidths(JTable table) {
				// strategy - get max width for cells in column and
				// make that the preferred width
		TableColumnModel columnModel = table.getColumnModel();
		int rowLen = table.getRowCount();
		
		
		for (int col=0; col<table.getColumnCount(); col++) {
		int maxwidth = 0;   
		         
		for (int row=0; row<table.getRowCount(); row++) {
			TableCellRenderer rend =table.getCellRenderer(row, col); 
			Object value = table.getValueAt (row, col); 
			Component comp =rend.getTableCellRendererComponent (table, value, false, false, row,col);
			maxwidth = Math.max (comp.getPreferredSize().width, maxwidth); 
			} // for row
			
			//TableColumn column = columnModel.getColumn (col); 
			//column.setPreferredWidth (maxwidth);

			
			TableColumn column = columnModel.getColumn (col);
			TableCellRenderer headerRenderer = column.getHeaderRenderer();
			if (headerRenderer == null)
		
			headerRenderer = table.getTableHeader().getDefaultRenderer();
			Object headerValue = column.getHeaderValue();
			Component headerComp =
					headerRenderer.getTableCellRendererComponent (table,
											  headerValue,
											  false,
											  false,
											  0,
											  col);
			maxwidth = Math.max (maxwidth,headerComp.getPreferredSize().width)+6;
			column.setPreferredWidth (maxwidth);

		} // for col 
				
			
			table.setRowHeight(27);
			
		
			}



	public void run() {
		TableUtil.redimTable(jTable, pane);
	}	
}
