package it.finanze.dogane.accise.ui.validation;

import java.math.BigInteger;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * Utilizzato per la validazione dei JTextField che devono contenere solo dati numerici positivi (senza segno)
 */
public class BigIntegerDocument extends PlainDocument {
	
	private int maxDim = 0;

	public BigIntegerDocument(int maxDim) {
		super();
		
		if(maxDim < 0) {
			throw new IllegalArgumentException("La dimensione massima deve essere almeno 0. Valore: " + maxDim);
		}
		
		this.maxDim = maxDim;
	}

	public void insertString(
		int offset,
		String string,
		AttributeSet attributes)
		throws BadLocationException {

		if (string == null) {
			return;
		} else {
			String newValue;
			int length = getLength();
			if (length == 0) {
				newValue = string;
			} else {
				String currentContent = getText(0, length);
				StringBuffer currentBuffer = new StringBuffer(currentContent);
				currentBuffer.insert(offset, string);
				newValue = currentBuffer.toString();
			}
			try {
				BigInteger bigInteger = new BigInteger(newValue);
				if(newValue.startsWith("+") || newValue.startsWith("-")) {
					return;
				}
				// Questo Test serve ad evitare che si digitino pi caratteri di quelli permessi
				// Ovviamente in caso di copia ed incolla con questo test o si copia tutta la stringa o niente!
				if (newValue.length() <= maxDim) {
					super.insertString(offset, string, attributes);
				}
			} catch (NumberFormatException exception) {
			}
		}
	}
}

