package it.finanze.dogane.accise.ui.validation;

import java.math.BigDecimal;
import java.util.NoSuchElementException;
import java.util.StringTokenizer;

import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import javax.swing.text.PlainDocument;

/**
 * Utilizzato per la validazione dei JTextField che devono contenere solo dati numerici (permesso il segno)
 */
public class SignedBigDecimalDocument extends PlainDocument {
	
	private int maxDimInt = 0;
	private int maxDimDec = 0;

	public SignedBigDecimalDocument(int maxDimInt, int maxDimDec) {
		super();
		
		if(maxDimInt <= 0) {
			throw new IllegalArgumentException("La dimensione massima per la parte intera deve essere un valore positivo. Valore: " + maxDimInt);
		}
		
		if(maxDimDec < 0) {
			throw new IllegalArgumentException("La dimensione massima per la parte decimale deve essere almeno 0. Valore: " + maxDimDec);
		}
		
		this.maxDimInt = maxDimInt;
		this.maxDimDec = maxDimDec;
	}

	public void insertString(
		int offset,
		String string,
		AttributeSet attributes)
		throws BadLocationException {

		if (string == null) {
			return;
		} else {
			String newValue;
			int length = getLength();
			if (length == 0) {
				newValue = string;
			} else {
				String currentContent = getText(0, length);
				StringBuffer currentBuffer = new StringBuffer(currentContent);
				currentBuffer.insert(offset, string);
				newValue = currentBuffer.toString();
			}
			try {
				if(newValue.equals("-")) {
					super.insertString(offset, string, attributes);
				} else {
					
					// Quindi secondo le specifiche di Java il separatore  il punto a meno di utilizzare
					// un NumberFormat diverso.
					BigDecimal bigDecimal = new BigDecimal(newValue);
					 
					if (isDimensionValid(newValue)) {
						super.insertString(offset, string, attributes);
					}
				}
			} catch (NumberFormatException exception) {
			}
		}
	}

	public void remove(int offs, int len) throws BadLocationException {

		int currentlen = getLength();
		String currentContent = getText(0, currentlen);
		StringBuffer currentBuffer = new StringBuffer(currentContent);
		
		currentBuffer.delete(offs, offs+len);
		String newValue = currentBuffer.toString();

		if(isDimensionValid(newValue)) {
			super.remove(offs, len);
		}
	}

	private boolean isDimensionValid(String value) {
		if(value.startsWith("-")) {
			value = value.substring(1);
		}
		boolean retvalue = false;
		StringTokenizer tokenizer = new StringTokenizer(value, ",."); // Ci metto pure la virgola hai visto mai!
		int lunghezzaParteIntera = 0;
		int lunghezzaParteDecimale = 0;
		try {
			lunghezzaParteIntera = tokenizer.nextToken().length();
			lunghezzaParteDecimale = tokenizer.nextToken().length();
		}
		catch(NoSuchElementException e) {
		}
				
		// Questo Test serve ad evitare che si digitino pi caratteri di quelli permessi
		// Ovviamente in caso di copia ed incolla con questo test o si copia tutta la stringa o niente!
		if (lunghezzaParteIntera <= maxDimInt && lunghezzaParteDecimale <= maxDimDec) {
			retvalue = true;
		}
		return retvalue;
	}

}
